/* ----------------------------------------------------------------------------
 *     Copyright (c) 2000-2021, Nexus Electronics Ltd (www.nexus.co.uk)
 *                            All rights reserved
 *
 * Redistribution and use in source and derived forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *  * Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *  * Redistributions in derived forms must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *  * Neither the name of Nexus Electronics Ltd. nor the names of its
 *    contributors may be used to endorse or promote products derived from
 *    this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY NEXUS ELECTRONICS LTD. "AS IS" AND ANY EXPRESS
 * OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL NEXUS ELECTRONICS LTD. BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * ----------------------------------------------------------------------------
 * Title       : triple-DES, single round
 * Project     : MAS
 * RCS ID      : 
 * $Id: desround.v,v 1.1.1.1 2001/01/09 13:02:21 el Exp $
 * ----------------------------------------------------------------------------
 * File        : desround.v
 * Author      : E.M. Lavelle
 * Company     : Nexus Electronics Ltd.
 * Last saved  : 22/08/2000
 * Version     : $Revision: 1111 $
 * ----------------------------------------------------------------------------
 * Description : 

 * the major part of the DES round processing. this module carries out the
 * expansion permutation, xors the result with the partial key input, runs
 * the resulting data through the sboxes, and then carries out the pbox
 * permutation. the sboxes are instantiated as ROM modules. the module is
 * purely combinatorial, and produces the PBOX output given the R4_RIGHT and
 * PRTL_KEY inputs.
 *  
 * ----------------------------------------------------------------------------
 * Modification history:
 * Date        Version Author     Comment
 * 22/08/2000  0.1      EML       created
 *
 * ------------------------------------------------------------------------- */

module desround(R4_RIGHT, PRTL_KEY, PBOX);
   input  [1:48] PRTL_KEY;
   input  [1:32] R4_RIGHT;
   output [1:32] PBOX;
   
   wire [1:48] 	 XOR1;
   wire [1:32] 	 SBOX;
   reg [1:48] 	 EBOX;

   // expansion permutation
   always @(R4_RIGHT) begin: expansion
      integer dst_index, i,j;
      
      dst_index = 2;
      for(i=0; i<8; i=i+1) begin
	 if(i==0)
	   EBOX[48] = R4_RIGHT[1];
	 else
	   EBOX[dst_index-2] = R4_RIGHT[(i*4)+1];
	 for(j=0; j<4; j=j+1)
	   EBOX[dst_index+j] = R4_RIGHT[(i*4)+1+j];
	 if(dst_index==44)
	   EBOX[1] = R4_RIGHT[(i+1)*4];
	 else
	   EBOX[dst_index+5] = R4_RIGHT[(i+1)*4];
	 dst_index = dst_index + 6;
      end
   end

   // xor the expansion permutation output with the current key
   assign XOR1 = PRTL_KEY ^ EBOX;

   /* sbox permutation. the 48-bit XOR1 is divided into 8 groups of 6 bits, 
    * and each of these forms the 6-bit address into a 64x4 ROM. the eight 
    * 4-bit outputs are combined to give the 32-bit result, SBOX.
    */
   rom_sbox1 ROM1(.A({XOR1[1],  XOR1[6],  XOR1[ 2: 5]}), .SPO(SBOX[ 1: 4]));
   rom_sbox2 ROM2(.A({XOR1[7],  XOR1[12], XOR1[ 8:11]}), .SPO(SBOX[ 5: 8]));
   rom_sbox3 ROM3(.A({XOR1[13], XOR1[18], XOR1[14:17]}), .SPO(SBOX[ 9:12]));
   rom_sbox4 ROM4(.A({XOR1[19], XOR1[24], XOR1[20:23]}), .SPO(SBOX[13:16]));
   rom_sbox5 ROM5(.A({XOR1[25], XOR1[30], XOR1[26:29]}), .SPO(SBOX[17:20]));
   rom_sbox6 ROM6(.A({XOR1[31], XOR1[36], XOR1[32:35]}), .SPO(SBOX[21:24]));
   rom_sbox7 ROM7(.A({XOR1[37], XOR1[42], XOR1[38:41]}), .SPO(SBOX[25:28]));
   rom_sbox8 ROM8(.A({XOR1[43], XOR1[48], XOR1[44:47]}), .SPO(SBOX[29:32]));

   // P-box permutation
   assign PBOX = {SBOX[16], SBOX[7],  SBOX[20], SBOX[21],
		  SBOX[29], SBOX[12], SBOX[28], SBOX[17],  
		  SBOX[1],  SBOX[15], SBOX[23], SBOX[26],  
		  SBOX[5],  SBOX[18], SBOX[31], SBOX[10],  
		  SBOX[2],  SBOX[8],  SBOX[24], SBOX[14],  
		  SBOX[32], SBOX[27], SBOX[3],  SBOX[9],  
		  SBOX[19], SBOX[13], SBOX[30], SBOX[6],  
		  SBOX[22], SBOX[11], SBOX[4],  SBOX[25]};

endmodule // des

/*----------------------------------- EOF -----------------------------------*/
